@@ -4,10 +4,13 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: sio.c,v 1.3 1995/09/10 16:40:28 chuck Exp $" ;
+static char RCSid[] = "$Id: sio.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 static char sio_version[] = VERSION ;
 
+#ifndef linux
 #include <sys/types.h>
+#endif
+
 #include <sys/stat.h>
 
 #include "sio.h"
@@ -17,10 +20,6 @@ static char sio_version[] = VERSION ;
 #include "events.h"
 #endif
 
-#if defined(__FreeBSD__) || defined(__bsdi__) || defined(__NetBSD__)
-PRIVATE char *sio_memscan( char *from, int how_many, register char ch ) ;
-#endif
-
 /*
  * SIO WRITE FUNCTIONS: Swrite, Sputc
  */
@@ -234,7 +233,7 @@ char *Srdline( fd )
 	IO_SETUP( fd, dp, __SIO_INPUT_STREAM, NULL ) ;
 	ASSERT( idp->start <= idp->nextb && idp->nextb <= idp->end ) ;
 
-#ifdef HAS_MMAP
+#ifdef HAVE_MMAP
 	if ( idp->memory_mapped && __sio_switch( idp, fd ) == FAILURE )
 		return( NULL ) ;
 #endif
