@@ -9,10 +9,10 @@
 #define __STR_H
 
 /*
- * $Id: str.h,v 3.1 1993/06/13 02:47:14 panos Exp $
+ * $Id: str.h,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $
  */
 
-#include <varargs.h>
+#include <stdarg.h>
 
 
 #ifdef __ARGS
@@ -29,17 +29,17 @@
 /*
  * strprint(3) functions
  */
-char *str_sprint __ARGS( ( char *buf, char *fmt, ... ) ) ;
-int str_nprint __ARGS( ( char *buf, char *fmt, ... ) ) ;
-void str_print __ARGS( ( int *count, char *buf, char *fmt, ... ) ) ;
+char *str_sprint ( char *buf, char *fmt, ... ) ;
+int str_nprint ( char *buf, char *fmt, ... ) ;
+void str_print ( int *count, char *buf, char *fmt, ... ) ;
 
 char *str_sprintv __ARGS( ( char *buf, char *fmt, va_list ) ) ;
 int str_nprintv __ARGS( ( char *buf, char *fmt, va_list ) ) ;
 void str_printv __ARGS( ( int *count, char *buf, char *fmt, va_list ) ) ;
 
-char *strx_sprint __ARGS( ( char *buf, int len, char *fmt, ... ) ) ;
-int strx_nprint __ARGS( ( char *buf, int len, char *fmt, ... ) ) ;
-void strx_print __ARGS( ( int *count, char *buf, int len, char *fmt, ... ) ) ;
+char *strx_sprint ( char *buf, int len, char *fmt, ... ) ;
+int strx_nprint ( char *buf, int len, char *fmt, ... ) ;
+void strx_print ( int *count, char *buf, int len, char *fmt, ... ) ;
 
 char *strx_sprintv __ARGS( ( char *buf, int len, char *fmt, va_list ) ) ;
 int strx_nprintv __ARGS( ( char *buf, int len, char *fmt, va_list ) ) ;
@@ -130,8 +130,8 @@ char *str_upper __ARGS( ( char *s ) ) ;
 
 typedef void *strs_h ;
 
-char *strs_search __ARGS( ( int flags, char *str, int len, char *pat, ... ) ) ;
-strs_h strs_setup __ARGS( ( int flags, char *pattern, ... ) ) ;
+char *strs_search ( int flags, char *str, int len, char *pat, ... ) ;
+strs_h strs_setup ( int flags, char *pattern, ... ) ;
 char *strs_match	__ARGS( ( strs_h handle, char *str, int len ) ) ;
 void strs_done		__ARGS( ( strs_h handle ) ) ;
 
