@@ -4,25 +4,22 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: strparse.c,v 3.1 1993/06/13 02:48:19 panos Exp $" ;
+static char RCSid[] = "$Id: strparse.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 static char version[] = VERSION ;
 
+#include <string.h>
+#include <stdlib.h>
+
 #include "str.h"
 #include "strparse.h"
 
-char *strcpy() ;
-char *strncpy() ;
-char *strpbrk() ;
-
-char *malloc() ;
-
 int str_errno ;
 
 
 PRIVATE char *new_string( s )
 	register char *s ;
 {
-	register char *p = malloc( (unsigned)strlen( s ) + 1 ) ;
+	register char *p = (char *) malloc( (unsigned)strlen( s ) + 1 ) ;
 
 	return( p ? strcpy( p, s ) : p ) ;
 }
