@@ -190,8 +190,8 @@ PRIVATE int filelog_write( xp, buf, len, flags, ap )
 
 	(void) time( &current_time ) ;
 	tmp = localtime( &current_time ) ;
-	cc = Sprint( flp->fl_fd, "%d/%d/%d@%02d:%02d:%02d",
-							tmp->tm_year, tmp->tm_mon+1, tmp->tm_mday,
+	cc = Sprint( flp->fl_fd, "%02d/%d/%d@%02d:%02d:%02d",
+							tmp->tm_year%100, tmp->tm_mon+1, tmp->tm_mday,
 							tmp->tm_hour, tmp->tm_min, tmp->tm_sec ) ;
 	msglen += cc ;
 
