@@ -4,17 +4,20 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: util.c,v 2.1 1993/05/06 07:40:12 panos Exp $" ;
+static char RCSid[] = "$Id: util.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
-extern int sys_nerr;
-extern char *sys_errlist[];
-extern int errno;
-
-char *malloc() ;
+#include <string.h>
+#include <stdlib.h>
+#include <stdio.h>
+#include <errno.h>
 
+#ifndef NUL
 #define NUL						'\0'
-#define NULL					0
+#endif
 
+#ifndef NULL
+#define NULL					0
+#endif
 
 /*
  * Search the given buffer for an occurrence of "%m"
@@ -38,12 +41,13 @@ char *__xlog_explain_errno( buf, size )
 	unsigned *size ;
 {
 	register int len ;
-	char *strncpy() ;
 
+	strx_nprint(buf, *size, "%s (errno = %d)", strerror(errno), errno);
+#if 0
 	if ( errno < sys_nerr )
 	{
 
-		(void) strncpy( buf, sys_errlist[ errno ], (int)*size ) ;
+		(void) strncpy( buf, strerror( errno ), (int)*size ) ;
 		for ( len = 0 ; len < *size ; len++ )
 			if ( buf[ len ] == NUL )
 				break ;
@@ -51,6 +55,7 @@ char *__xlog_explain_errno( buf, size )
 	}
 	else
 		len = strx_nprint( buf, *size, "errno = %d", errno ) ;
+#endif
 	return( buf ) ;
 }
 
@@ -60,7 +65,6 @@ char *__xlog_new_string( s )
 {
 	unsigned size = strlen( s ) + 1 ;
 	char *p = malloc( size ) ;
-	char *strcpy() ;
 
 	return( ( p != NULL ) ? strcpy( p, s ) : p ) ;
 }
