@@ -8,10 +8,10 @@ static char RCSid[] = "$Id: access.c,v 1.3 1999/10/13 02:51:03 bbraun Exp $" ;
 
 #include <syslog.h>
 #include <time.h>
+#include <signal.h>
 
 #ifdef LIBWRAP
 #include <tcpd.h>
-#include <signal.h>
 int deny_severity = LOG_INFO;
 int allow_severity = LOG_INFO;
 #endif
@@ -41,10 +41,38 @@ struct name_value access_code_names[] =
 		{ "per_source_limit",	(int) AC_PER_SOURCE_LIMIT	},
 		{ "process_limit",		(int) AC_PROCESS_LIMIT	},
 		{ "libwrap",            (int) AC_LIBWRAP        },
+      { "load",               (int) AC_LOAD           },
+      { "connections per second", (int) AC_CPS        },
 		{ CHAR_NULL,				1								},
 		{ "UNKNOWN",				0								}
 	} ;
 
+/* Signal handler for re-enabling a service that has been disabled. */
+PRIVATE void alrm_ena()
+{
+	int i;
+	time_t nowtime;
+
+	nowtime = time(NULL);
+	for( i=0; i < pset_count( SERVICES(ps) ); i++ ) {
+		struct service *sp;
+		struct service_config *scp;
+
+		sp = pset_pointer( SERVICES(ps), i);
+		scp = SVC_CONF( sp );
+
+		if( sp->svc_state == SVC_DISABLED ) {
+			if( scp->sc_time_reenable <= nowtime ) {
+				/* re-enable the service */
+				if( svc_activate(sp) == OK ) {
+					syslog(LOG_ERR, "Activating service %s\n", scp->sc_name);
+				} else {
+					syslog(LOG_ERR, "Error activating service %s\n", scp->sc_name);
+				} /* else */
+			}
+		}
+	} /* for */
+} /* alrm_ena */
 
 /*
  * Returns OK if the IP address in sinp is acceptable to the access control
@@ -52,11 +80,27 @@ struct name_value access_code_names[] =
  */
 PRIVATE status_e remote_address_check( sp, sinp )
 	register struct service		*sp ;
-	struct sockaddr_in			*sinp ;
+#ifdef INET6
+	struct sockaddr_in6 		*sinp ;
+#else
+	struct sockaddr_in 			*sinp ;
+#endif
 {
 	/*
 	 * of means only_from, na means no_access
 	 */
+#ifdef INET6
+	struct in6_addr         of_match ;
+	struct in6_addr         na_match ;
+	register bool_int		of_matched  = 0;
+	register bool_int		na_matched  = 0;
+
+	struct in6_addr			*addr ;
+
+	bzero(&of_match, sizeof(of_match));
+	bzero(&na_match, sizeof(na_match));
+	addr = &sinp->sin6_addr ;
+#else
 	unsigned long			of_match ;
 	unsigned long			na_match ;
 	register bool_int		of_matched ;
@@ -64,14 +108,20 @@ PRIVATE status_e remote_address_check( sp, sinp )
 	struct in_addr			*addr ;
 
 	addr = &sinp->sin_addr ;
+#endif
 
 	if ( sp->svc_no_access != NULL )
 		na_matched = addrlist_match( sp->svc_no_access, addr, &na_match ) ;
+	else if ( na_matched == FERROR )
+		/* There was an error matching the addresses */
+		return FAILED;
 	else
 		na_matched = FALSE ;
 
 	if ( sp->svc_only_from != NULL )
 		of_matched = addrlist_match( sp->svc_only_from, addr, &of_match ) ;
+	else if ( of_matched == FERROR )
+		return FAILED;
 	else
 		of_matched = FALSE ;
 
@@ -86,17 +136,31 @@ PRIVATE status_e remote_address_check( sp, sinp )
 		 * and we cannot allow a server to start.
 		 * We do not disable the service entry (not our job).
 		 */
+#ifdef INET6
+		int ret = memcmp(&na_match, &of_match, sizeof(na_match));
+	
+		if( ret == 0 )
+			msg( LOG_ERR, "remote_address_check",
+"Service=%s: only_from list and no_access list match equally the address %s",
+				SVC_ID( sp ), xntoa( *sinp ) ) ;
+		else if ( ret < 0 )
+			return OK;
+		else
+			return FAILED;
+#else
 		if ( na_match == of_match )
 			msg( LOG_ERR, "remote_address_check",
 "Service=%s: only_from list and no_access list match equally the address %s",
-				SVC_ID( sp ), inet_ntoa( sinp->sin_addr ) ) ;
+				SVC_ID( sp ), xntoa( *sinp ) ) ;
 		return( ( of_match > na_match ) ? OK : FAILED ) ;
+#endif
 	}
 
 	if ( sp->svc_no_access != NULL && na_matched )
 		return( FAILED ) ;
-	if ( sp->svc_only_from != NULL && ! of_matched )
+	if ( sp->svc_only_from != NULL && ! of_matched ) {
 		return( FAILED ) ;
+	}
 
 	/*
 	 * If no lists were specified, the default is to allow starting a server
@@ -120,8 +184,13 @@ access_e access_control( sp, cp, check_mask )
 	register mask_t				*check_mask ;
 {
 	register struct service_config	*scp = SVC_CONF( sp ) ;
+#ifdef INET6
+	struct sockaddr_in6					*sinp = SOCKADDRIN_NULL;
+#else
 	struct sockaddr_in					*sinp = SOCKADDRIN_NULL;
+#endif
 	int u, n;
+	time_t nowtime;
 
 #ifdef LIBWRAP
    struct request_info req;
@@ -134,6 +203,32 @@ access_e access_control( sp, cp, check_mask )
       return(AC_LIBWRAP);
    }
 #endif
+	/* If cps is specified, try to detect someone bombarding us, and 
+	 * just disable the service.
+	 */
+	if( scp->sc_time_conn_max != 0 ) {
+		nowtime = time(NULL);
+		if( ((scp->sc_time_limit - nowtime) > 2) || ( (scp->sc_time_limit-nowtime) < 0)) {
+			scp->sc_time_limit = nowtime;
+			scp->sc_time_conn = 1;
+		} else {
+			scp->sc_time_conn ++;
+			if( scp->sc_time_conn > scp->sc_time_conn_max ) {
+				svc_deactivate(sp);
+				/* Need to implement a restart timer */
+				syslog(LOG_ERR, "Deactivating service %s due to excessive incoming connections.  Restarting in %d seconds.\n", scp->sc_name, scp->sc_time_wait);
+				if( signal( SIGALRM, alrm_ena) == SIG_ERR ) {
+					syslog(LOG_ERR, "Error setting up timer for reactivating service %s\n", scp->sc_name);
+					return(AC_CPS);
+				}
+				nowtime = time(NULL);
+				scp->sc_time_reenable = nowtime + scp->sc_time_wait;
+				alarm(scp->sc_time_wait);
+				return(AC_CPS);
+			}
+		}
+	}
+
 
 #ifdef HAVE_LOADAVG
 	if ( scp->sc_max_load != 0 ) {
@@ -174,8 +269,12 @@ access_e access_control( sp, cp, check_mask )
 				if ( (SERVER_SERVICE( serp ) == sp) &&
 					( cop = SERVER_CONNECTION( serp ) ) )
 				{
+#ifdef INET6
+					if ( IN6_ARE_ADDR_EQUAL( &(cop->co_remote_address.sin6_addr), &(sinp->sin6_addr) ) ) 
+#else
 					if ( cop->co_remote_address.sin_addr.s_addr ==
 						sinp->sin_addr.s_addr)
+#endif
 							n++;
 				}
 			}
