@@ -34,7 +34,8 @@ typedef enum
 		                     /* service and source address                   */
       AC_PROCESS_LIMIT,    /* total process limit would be exceeded        */
       AC_LIBWRAP,
-      AC_LOAD
+      AC_LOAD,
+      AC_CPS
    } access_e ;
 
 extern struct name_value access_code_names[] ;
