@@ -12,11 +12,15 @@ static char RCSid[] = "$Id: addr.c,v 1.2 1999/10/15 05:18:39 bbraun Exp $" ;
 #include <syslog.h>
 #include <netdb.h>
 #include <memory.h>
+#ifdef bsdi
+#include <arpa/inet.h>
+#endif
 
 #include "pset.h"
 
 #include "defs.h"
 #include "addr.h"
+#include "str.h"
 
 #if !defined(HAVE_SNPRINTF)
 extern int      snprintf(char *, size_t, const char *, ...);
@@ -48,8 +52,14 @@ typedef enum { CANT_PARSE, PARSED, ERROR } result_e ;
 struct comp_addr
 {
 	address_e		addr_type ;
+	char          *name ;
+#ifdef INET6
+	struct in6_addr addr ;
+	struct in6_addr mask ;
+#else
 	unsigned long	addr ;
 	unsigned long	mask ;
+#endif
 } ;
 
 #define CAP( p )        		( (struct comp_addr *) (p) )
@@ -66,43 +76,184 @@ struct comp_addr
  * address (a greater numerical value for the mask implies a more
  * precise match).
  */
+
+#ifdef INET6
+
+/* The addrlist_match function sets the mask for IPv6 addresses.
+ * mask is a pointer to the in6_addr structure, bits is the
+ * number of bits to set in the mask, and len is the length of mask.
+ */
+void xsetmask(char *mask, unsigned int bits, int len)
+{
+	int i;
+	int bytes = bits/8;
+	int remain = bits - (bytes * 8);
+	char *func = "xsetmask";
+	
+	bzero(mask, len);
+
+	/* This may be wrong for bigendian... */
+	for(i=0; i < bytes; i++ ) {
+		mask[i] = 0xFF;
+	}
+
+	if( remain > 0 )
+		mask[i] = ( 0xFF << (8-remain) );
+
+	return;
+}
+/* This is a helper function to make address matching with mask
+ * work ok w/ipv6 
+ * Returns TRUE if addr1&mask1 == addr2
+ */
+bool_int xmatch(char *addr1, char *mask1, char *addr2, int len)
+{
+	int i;
+
+	for(i=0; i < len; i++ ) {
+		if( (addr1[i] & mask1[i]) != addr2[i] ) {
+			return( FALSE );
+		}
+	}
+	return TRUE;
+} 
+
+/* maps a v4 address into a v6 address */
+void xaddrmap(char *v4, char *v6)
+{
+	if( (v4 == NULL) || (v6 == NULL) )
+		return;
+
+	strcpy(v6, "::ffff:");
+	strcat(v6,v4);
+
+	return;
+}
+
+bool_int addrlist_match( addr_list, addr, matchp ) 
+	pset_h			addr_list ; 
+	struct in6_addr *addr ; 
+	struct in6_addr	*matchp ; 
+{ 
+	struct in6_addr remote_addr;
+	unsigned u;
+
+	memcpy(&remote_addr, addr, sizeof(remote_addr));
+
+	for ( u = 0 ; u < pset_count( addr_list ) ; u++ ) {
+		struct comp_addr *cap = CAP( pset_pointer( addr_list, u ) ) ;
+
+		if( (cap->addr_type == HOST_ADDR) && (cap->name != NULL) ) {
+			struct hostent *hep;
+			char *hname;
+		
+			hep = gethostbyaddr((char *)addr, 16, AF_INET6);
+			if( hep == NULL ) {
+				if( memcmp(&remote_addr, &(cap->addr), sizeof(remote_addr)) == 0 )
+					return TRUE;
+				continue;
+			}
+
+			/* Parse the address as a domain portion */
+			if( cap->name[0] == '.' )
+			{
+				hname = str_casefind( hep->h_name, cap->name );
+				if( hname != NULL ) {
+					if( strlen(cap->name) == strlen(hname) ) 
+						return( TRUE );
+				}
+			} else {
+				if( str_casefind( hep->h_name, cap->name ) == hep->h_name ) 
+					return( TRUE );
+			}
+		} else {
+
+			if( xmatch( (char *)&remote_addr, (char *)&(cap->mask), (char *)&(cap->addr), sizeof(remote_addr)) == TRUE ) {
+				memcpy(matchp, &(cap->mask), sizeof(*matchp));
+				*matchp = cap->mask ;
+				return( TRUE );
+			}
+		}
+	}
+	return(FALSE);
+}
+
+#else /* INET6 */
+
 bool_int addrlist_match( addr_list, addr, matchp )
 	pset_h			addr_list ;
 	struct in_addr *addr ;
 	unsigned long	*matchp ;
 {
-	register unsigned long remote_addr = ntohl( addr->s_addr ) ;
+	unsigned long remote_addr = ntohl( addr->s_addr ) ;
 	register unsigned u ;
 
 	for ( u = 0 ; u < pset_count( addr_list ) ; u++ )
 	{
 		register struct comp_addr *cap = CAP( pset_pointer( addr_list, u ) ) ;
 
-		if ( ( remote_addr & cap->mask ) == cap->addr )
+		/* If it's a HOST_ADDR type, do a name lookup and see if they 
+		 * match.
+		 */
+		if( (cap->addr_type == HOST_ADDR) && (cap->name != NULL) )
 		{
-			*matchp = cap->mask ;
-			return( TRUE ) ;
+			struct hostent *hep;
+			char *hname;
+		
+			hep = gethostbyaddr((char *)addr, 4, AF_INET);
+			if( hep == NULL ) {
+				if( memcmp(&remote_addr, &(cap->addr), sizeof(remote_addr)) == 0 )
+					return TRUE;
+				continue;
+			}
+
+			/* Parse the address as a domain portion */
+			if( cap->name[0] == '.' )
+			{
+				hname = str_casefind( hep->h_name, cap->name );
+				if( hname != NULL ) {
+					if( strlen(cap->name) == strlen(hname) ) 
+						return( TRUE );
+				}
+			} else {
+				if( str_casefind( hep->h_name, cap->name ) == hep->h_name ) 
+					return( TRUE );
+			}
+		} else {
+
+			if ( ( remote_addr & cap->mask ) == cap->addr )
+			{
+				*matchp = cap->mask ;
+				return( TRUE ) ;
+			}
 		}
 	}
 	return( FALSE ) ;
 }
-
-
+#endif /* INET6 */
 
 void addrlist_dump( addr_list, fd )
 	pset_h	addr_list ;
 	int		fd ;
 {
 	register unsigned u ;
+#ifdef INET6
+	struct in6_addr inaddr ;
+#else
 	struct in_addr inaddr ;
-	char *inet_ntoa() ;
+#endif
 
 	for ( u = 0 ; u < pset_count( addr_list ) ; u++ )
 	{
 		struct comp_addr *cap = CAP( pset_pointer( addr_list, u ) ) ;
 		char *type ;
 
-		inaddr.s_addr = htonl( cap->addr ) ;
+#ifndef INET6
+		inaddr.s_addr = ntohl( cap->addr ) ;
+#else
+		memcpy(&inaddr, &(cap->addr), sizeof(cap->addr));
+#endif
+
 		if ( cap->addr_type == NUMERIC_ADDR )
 			type = "NUMERIC" ;
 		else if ( cap->addr_type == NET_ADDR )
@@ -112,7 +263,19 @@ void addrlist_dump( addr_list, fd )
 		else
 			type = "BAD" ;
 		
+/*
+#ifndef INET6
+		Sprint( fd, "foo");
+		Sprint( fd, " %s(%s)", xntoa( inaddr.s_addr ), type ) ;
+#else
+		Sprint( fd, " %s(%s)", xntoa( cap->addr ), type ) ;
+#endif
+*/
+#ifndef INET6
 		Sprint( fd, " %s(%s)", inet_ntoa( inaddr ), type ) ;
+#else
+		Sprint( fd, " %s(%s)", inet_ntop( inaddr ), type ) ;
+#endif
 	}
 }
 
@@ -173,7 +336,12 @@ PRIVATE status_e xremove( addr_list, cap )
 	{
 		old_cap = CAP( pset_pointer( addr_list, u ) ) ;
 
+#ifdef INET6
+		if ( IN6_ARE_ADDR_EQUAL( &(old_cap->addr), &(cap->addr)) && 
+			IN6_ARE_ADDR_EQUAL( &(old_cap->mask), &(cap->mask)) )
+#else
 		if ( old_cap->addr == cap->addr && old_cap->mask == cap->mask )
+#endif
 		{
 			pset_pointer( addr_list, u ) = NULL ;
 			FREE_CAP( old_cap ) ;
@@ -186,6 +354,8 @@ PRIVATE status_e xremove( addr_list, cap )
 /*
  * Function allows the use of 0.0.0.0/24 style address ranges for access cntl.
  *   --bbraun 10/26/98
+ *
+ * Updated to handle ::/46 style address ranges for access cntl.
  */
 PRIVATE result_e explicit_mask( char *str_addr, statfunc op, pset_h addr_list)
 {
@@ -194,6 +364,43 @@ PRIVATE result_e explicit_mask( char *str_addr, statfunc op, pset_h addr_list)
 	int val;
 	int a, b, c, d, mask;
 
+#ifdef INET6
+	char saddr[INET6_ADDRSTRLEN];
+	char v6addr[46];
+	bzero(saddr, INET6_ADDRSTRLEN);
+
+	/* Ugly hack, but sscanf doesn't like the '/' */
+	for( a=0; a < strlen(str_addr); a++ ) {
+		if( str_addr[a] == '/' )
+			str_addr[a] = ' ';
+	}
+	val = sscanf(str_addr, "%46s %d", saddr, &mask);
+
+	/* If v6 address fails, assume it is a v4 address, mapp it and try again */
+	if( inet_pton(AF_INET6, saddr, &ca.addr) != 1 ) {
+		bzero(v6addr, 46);
+		xaddrmap(saddr, v6addr);
+		if( inet_pton(AF_INET6, v6addr, &ca.addr) != 1 )
+			return(CANT_PARSE);
+
+		/* convert the v4 mask into a v6 mapped mask */
+		mask += 96;
+	}
+
+	if( val < 2 ) {
+		return(CANT_PARSE);
+	}
+
+	if( mask >= 128 ) {
+		ca.addr_type = HOST_ADDR;
+	} else {
+		ca.addr_type = NET_ADDR;
+	}
+
+	xsetmask((char *)&ca.mask, mask, sizeof(ca.mask));
+	
+	return( ( (*op)( addr_list, &ca ) == OK ) ? PARSED : ERROR ) ;
+#else
 	val = sscanf(str_addr, "%d.%d.%d.%d/%d", &a, &b, &c, &d, &mask);
 	if( val < 5 )
 	{
@@ -206,7 +413,7 @@ PRIVATE result_e explicit_mask( char *str_addr, statfunc op, pset_h addr_list)
 
 	snprintf(newaddr, 16, "%d.%d.%d.%d", a, b, c, d);
 
-	ca.addr = inet_addr(&newaddr);
+	ca.addr = inet_addr(newaddr);
 
 	if(mask == 32)
 	{
@@ -243,6 +450,7 @@ PRIVATE result_e explicit_mask( char *str_addr, statfunc op, pset_h addr_list)
 #endif
 	
 	return( ( (*op)( addr_list, &ca ) == OK ) ? PARSED : ERROR ) ;
+#endif /* INET6 */
 }
 
 /*
@@ -253,6 +461,7 @@ PRIVATE result_e net_addr( str_addr, op, addr_list )
 	statfunc		op ;
 	pset_h		addr_list ;
 {
+
 	/*
 	 *
 	 *  The following table demonstrates how the mask is determined
@@ -280,6 +489,10 @@ PRIVATE result_e net_addr( str_addr, op, addr_list )
 	int							i ;
 	char							*func = "net_addr" ;
 
+#ifdef INET6
+	/* Not yet implemented */
+	return(CANT_PARSE);
+#else
 	nep = getnetbyname( str_addr ) ;
 	if ( nep == NULL || nep->n_addrtype != AF_INET || nep->n_net == 0 )
 		return( CANT_PARSE ) ;
@@ -300,6 +513,7 @@ PRIVATE result_e net_addr( str_addr, op, addr_list )
 			return( ( (*op)( addr_list, &ca ) == OK ) ? PARSED : ERROR ) ;
 		}
 	}
+#endif /* INET6 */
 }
 
 
@@ -316,9 +530,14 @@ PRIVATE result_e numeric_addr( str_addr, op, addr_list )
 	register unsigned long mask ;
 	struct comp_addr ca ;
 
+#ifdef INET6
+	/* Not yet implemented */
+	return(CANT_PARSE);
+#else
 	addr = ntohl( inet_addr( str_addr ) ) ;
-	if ( addr == (unsigned long) -1 )
+	if ( addr == (unsigned long) -1 ) {
 		return( CANT_PARSE ) ;
+	}
 
 	if ( addr == 0 )
 		mask = 0 ;			/* i.e. matches everything */
@@ -337,6 +556,7 @@ PRIVATE result_e numeric_addr( str_addr, op, addr_list )
 	ca.addr = addr ;
 	ca.addr_type = NUMERIC_ADDR ;
 	return( ( (*op)( addr_list, &ca ) == OK ) ? PARSED : ERROR ) ;
+#endif /* INET6 */
 }
 
 
@@ -353,12 +573,72 @@ PRIVATE result_e host_addr( str_addr, op, addr_list )
 	struct hostent *hep ;
 	struct comp_addr ca ;
 	char **ap ;
+	char *func = "host_addr";
+
+#ifdef INET6
+	struct addrinfo hints, *res, *ressave;
+	struct sockaddr_in6 remsa;
+	char v6addr[46];
+	int n;
+	char *inet_ntoa(struct in_addr);
+
+	bzero(&hints, sizeof(hints));
+	hints.ai_flags = AI_CANONNAME;
+	hints.ai_family = AF_INET6;
+
+	if( (n = getaddrinfo(str_addr, NULL, &hints, &res)) < 0 ) {
+		hints.ai_family = AF_INET;
+
+		if( (n = getaddrinfo(str_addr, NULL, &hints, &res)) < 0 ) 
+			return CANT_PARSE;
+
+		bzero(v6addr, 46);
+		hints.ai_family = AF_INET6;
+		xaddrmap( inet_ntoa(((struct sockaddr_in *)(res->ai_addr))->sin_addr), v6addr);
+		freeaddrinfo(res);
+		if( (n = getaddrinfo(v6addr, NULL, &hints, &res)) < 0 )
+			return( CANT_PARSE );
+	}
+
+	ca.addr_type = HOST_ADDR ;
+
+	ressave = res;
+	while( res != NULL ) {
+
+		
+		memcpy(&(ca.addr), &((struct sockaddr_in6 *)(res->ai_addr))->sin6_addr, 
+			sizeof(ca.addr));
+		xsetmask((char*)&(ca.mask), 128, sizeof(ca.mask));
+
+		if ( (*op)( addr_list, &ca ) == FAILED ) {
+			freeaddrinfo(res);
+			return( ERROR ) ;
+		}
+	
+		res = res->ai_next;
+	}
+
+	freeaddrinfo(res);
+	return( PARSED );
+#else
+	if( str_addr[0] == '.' ) {
+		ca.addr_type = HOST_ADDR;
+		ca.name = str_addr;
+		if ( (*op)( addr_list, &ca ) == FAILED )
+			return( ERROR ) ;
+		return( PARSED );
+	}
 
 	hep = gethostbyname( str_addr ) ;
 	if ( hep == NULL || hep->h_addrtype != AF_INET )
 		return( CANT_PARSE ) ;
 
 	ca.addr_type = HOST_ADDR ;
+	ca.name = str_addr;
+	if( isdigit(str_addr[strlen(str_addr)-1]) ) {
+		ca.addr_type = NUMERIC_ADDR;
+	}
+
 	for ( ap = hep->h_addr_list ; *ap ; ap++ )
 	{
 		struct in_addr inaddr ;
@@ -368,12 +648,13 @@ PRIVATE result_e host_addr( str_addr, op, addr_list )
 		 */
 		(void) memcpy( (char *) &inaddr, *ap, hep->h_length ) ;
 
-		ca.addr = ntohl( inaddr.s_addr ) ;
+		ca.addr = ntohl( inaddr.s_addr ) ; 
 		ca.mask = 0xFFFFFFFF ;
 		if ( (*op)( addr_list, &ca ) == FAILED )
 			return( ERROR ) ;
 	}
 	return( PARSED ) ;
+#endif
 }
 
 
@@ -399,6 +680,11 @@ PRIVATE result_e factorized_addr( str_addr, op, addr_list )
 	struct comp_addr	ca ;
 	char					*func = "factorized_addr" ;
 
+#ifdef INET6
+	/* Not implemented */
+	return(CANT_PARSE);
+#else
+
 	for ( p = str_addr ; *p != OPEN_CURLY_BRACKET ; last = *p++ )
 	{
 		if ( isdigit( *p ) )
@@ -484,6 +770,7 @@ PRIVATE result_e factorized_addr( str_addr, op, addr_list )
 		}
 	}
 	/* NOTREACHED */
+#endif /* INET6 */
 }
 
 
@@ -503,11 +790,11 @@ PRIVATE status_e addrlist_op( addr_list, op, str_addr )
 	int i ;
 	static result_e (*addr_parser[])() =
 		{
-			explicit_mask,
 			numeric_addr,
+			host_addr,
+			explicit_mask,
 			factorized_addr,
 			net_addr,
-			host_addr,
 			NULL
 		} ;
 	char *func = "addrlist_op" ;
