@@ -46,14 +46,16 @@
 #define A_BANNER_SUCCESS   29
 #define A_BANNER_FAIL      30
 #define A_MAX_LOAD         31
+#define A_CPS              32
 
 /*
  * SERVICE_ATTRIBUTES is the number of service attributes and also
  * the number from which defaults-only attributes start.
  */
-#define SERVICE_ATTRIBUTES		( A_NICE + 1 )
+#define SERVICE_ATTRIBUTES		( A_CPS + 1 )
 
 #define A_DISABLED			( SERVICE_ATTRIBUTES )
+#define A_ENABLED                       ( A_DISABLED + 1 )
 
 
 /*
