@@ -237,7 +237,7 @@ PRIVATE void dgram_daytime( serp )
 }
 
 
-#define TIME_OFFSET			220898880
+#define TIME_OFFSET			2208988800UL
 
 /*
  * We always report the time using network-byte-order
@@ -509,27 +509,45 @@ PRIVATE void xadmin_handler( struct server *serp )
 			  for( u = 0 ; u < pset_count( SERVICES( ps ) ) ; u++ )
 			  {
 				  struct service *sp=NULL;
-				  unsigned long baddr=0;
+#ifdef INET6
+                  struct sockaddr_in6 baddr;
+                  struct sockaddr_in6 raddr;
+#else
+                  struct sockaddr_in baddr;
+                  struct sockaddr_in raddr;
+#endif
+
+                  bzero(&baddr, sizeof(baddr));
+                  bzero(&raddr, sizeof(raddr));
 
 				  sp = SP( pset_pointer( SERVICES( ps ), u ) );
 
 				  if( SVC_CONF(sp)->sc_bind_addr != NULL )
-				    baddr = SVC_CONF(sp)->sc_bind_addr->s_addr;
+#ifdef INET6
+                    memcpy(&baddr.sin6_addr, &SVC_CONF(sp)->sc_bind_addr, sizeof(baddr));
+#else
+                    memcpy(&baddr.sin_addr, &SVC_CONF(sp)->sc_bind_addr, sizeof(baddr));
+#endif
   
 				 if( SVC_CONF(sp)->sc_redir_addr != NULL )
 				 {
+#ifdef INET6
+                    memcpy(&raddr.sin6_addr, &SVC_CONF(sp)->sc_bind_addr, sizeof(raddr));
+#else
+                    memcpy(&raddr.sin_addr, &SVC_CONF(sp)->sc_bind_addr, sizeof(raddr));
+#endif
 				   Sprint(descriptor, "%-10s ", SC_NAME( SVC_CONF( sp ) ) );
 				   Sprint(descriptor, "%-6d ", SC_PORT( SVC_CONF( sp ) ) );
-				   Sprint(descriptor, "%-16s ", inet_ntoa( baddr ) );
+				   Sprint(descriptor, "%-16s ", xntoa( baddr ) );
 				   Sprint(descriptor, "%-6d ", SVC_CONF(sp)->sc_uid );
-				   Sprint(descriptor, "%-16s ", inet_ntoa(SVC_CONF(sp)->sc_redir_addr->s_addr) );
+				   Sprint(descriptor, "%-16s ", xntoa(raddr) );
 				   Sprint(descriptor, "%-6d\n", SVC_CONF(sp)->sc_redir_port );
 				 }
 				 else
 				 {
 				   Sprint(descriptor, "%-10s ", SC_NAME( SVC_CONF( sp ) ) );
 				   Sprint(descriptor, "%-6d ", SC_PORT( SVC_CONF( sp ) ) );
-				   Sprint(descriptor, "%-16s ", inet_ntoa( baddr ) );
+				   Sprint(descriptor, "%-16s ", xntoa( baddr ) );
 				   Sprint(descriptor, "%-6d\n", SVC_CONF(sp)->sc_uid );
 				 }
 
