@@ -82,11 +82,15 @@ PRIVATE void exec_server( serp )
 
 	(void) close( descriptor ) ;
 
+#ifdef solaris
 	if ( debug.on )
 	{
 		msg( LOG_DEBUG, func, "exec( %s ). no control terminal", server ) ;
 		no_control_tty() ;
 	}
+#else
+	no_control_tty() ;
+#endif
 
 #ifdef RLIMIT_NOFILE
 	{
@@ -196,6 +200,7 @@ PRIVATE void set_credentials( scp )
 				if ( setgroups( 0, NULL ) )
 				{
 					msg( LOG_ERR, func, "setgroups( 0, NULL ) failed: %m" ) ;
+					msg( LOG_ERR, func, "Your system may require that 'groups = yes' be defined for this service: %s", scp->sc_name);
 					_exit( 1 ) ;
 				}
 			}
@@ -281,8 +286,10 @@ void child_process( serp )
 		}
 		else
 		{
+#ifndef INET6
 #if defined(HAVE_SETENV)
 			setenv("REMOTE_HOST", inet_ntoa(cp->co_remote_address.sin_addr), 0);
+#endif
 #endif
 			exec_server( serp ) ;
 		}
@@ -304,13 +311,23 @@ void child_process( serp )
 		else
 		{
 			int namelen = sizeof( name ) - 1 ;		/* leave space for the NUL */
+#ifdef INET6
+			struct sockaddr_in6 *sinp = conn_address( SERVER_CONNECTION( serp ) ) ;
+#else
 			struct sockaddr_in *sinp = conn_address( SERVER_CONNECTION( serp ) ) ;
+#endif
 			int len = strx_nprint( name, namelen,
 										"(%s service) %s", program_name, SC_ID( scp ) ) ;
 			
+#ifdef INET6
+			if ( SC_ACCEPTS_CONNECTIONS( scp ) && !IN6_IS_ADDR_UNSPECIFIED(sinp) )
+				strx_print( INT_NULL, &name[ len ], namelen - len,
+													" %s" , xntoa( *sinp ) ) ;
+#else
 			if ( SC_ACCEPTS_CONNECTIONS( scp ) && sinp != SOCKADDRIN_NULL )
 				strx_print( INT_NULL, &name[ len ], namelen - len,
 													" %s", inet_ntoa( sinp->sin_addr ) ) ;
+#endif
 		}
 		rename_process( name ) ;
 		svc_internal( sp, serp ) ;
