@@ -4,12 +4,10 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: conf.c,v 1.3 1995/09/10 18:41:13 chuck Exp $" ;
+static char RCSid[] = "$Id: conf.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include <sys/types.h>
-#if defined(BSD) || defined(linux)
 #include <netinet/in.h>
-#endif
 #include <syslog.h>
 #include <fcntl.h>
 #include <string.h>
