@@ -239,6 +239,7 @@ void conn_dump( cp, fd )
 	register connection_s *cp ;
 	int fd ;
 {
+#ifndef INET6
 	register unsigned u ;
 	void tabprint() ;
 
@@ -258,5 +259,6 @@ void conn_dump( cp, fd )
 	if ( cp->co_alternative_count > 0 )
 		tabprint( fd, 1, "next alternative service = %s\n",
 				SVC_ID( cp->co_alternatives[ cp->co_next_alternative ] ) ) ;
+#endif
 }
 
