@@ -13,10 +13,19 @@
 
 #include <sys/types.h>
 #include <netinet/in.h>
+#ifdef bsdi
+#include <arpa/inet.h>
+#endif
 
 #include "mask.h"
 #include "service.h"
 
+#ifdef INET6
+char *xntoa(struct sockaddr_in6);
+#else
+char *xntoa(struct sockaddr_in);
+#endif
+
 #define MAX_ALTERNATIVES				3
 
 typedef enum { CONN_CLOSED = 0, CONN_OPEN } conn_state_e ;
@@ -32,7 +41,11 @@ struct connection
 	struct service 		*co_sp ;
 	int 						co_descriptor ;
 	mask_t 					co_flags ;
+#ifdef INET6
+	struct sockaddr_in6 co_remote_address ;
+#else
 	struct sockaddr_in	co_remote_address ;
+#endif
 	unsigned 				co_alternative_count ;
 	unsigned 				co_next_alternative ;
 	struct service 		*co_alternatives[ MAX_ALTERNATIVES ] ;
@@ -56,24 +69,36 @@ typedef struct connection connection_s ;
 #define conn_cleanup( cp )				conn_set_flag( cp, COF_CLEANUP )
 
 #define conn_setaddr( cp, sinp )															\
-							{																		\
-								conn_set_flag( cp, COF_HAVE_ADDRESS ) ;				\
-								(cp)->co_remote_address = *(sinp) ;						\
-							}
+		{	\
+			conn_set_flag( cp, COF_HAVE_ADDRESS ) ;		\
+			memcpy(&((cp)->co_remote_address), sinp, sizeof(*sinp) ); \
+		}
+/*
+			(cp)->co_remote_address = *(sinp) ;			
+*/
 #define conn_set_descriptor( cp, fd )		(cp)->co_descriptor = (fd)
 
+#ifdef INET6
 #define conn_address( cp )																	\
-									(																\
-										M_IS_SET( (cp)->co_flags, COF_HAVE_ADDRESS )	\
-													? &(cp)->co_remote_address 			\
-													: SOCKADDRIN_NULL							\
-									)
+		(	\
+			M_IS_SET( (cp)->co_flags, COF_HAVE_ADDRESS )	\
+				? (struct sockaddr_in6 *)&((cp)->co_remote_address) 	\
+				: SOCKADDRIN_NULL							\
+		)
+#else
+#define conn_address( cp )																	\
+		(	\
+			M_IS_SET( (cp)->co_flags, COF_HAVE_ADDRESS )	\
+				? &((cp)->co_remote_address) 	\
+				: SOCKADDRIN_NULL							\
+		)
+#endif
 
 char *inet_ntoa() ;
 
 #define conn_addrstr( cp )		\
 	M_IS_SET( (cp)->co_flags, COF_HAVE_ADDRESS ) \
-		? xntoa( (cp)->co_remote_address.sin_addr.s_addr )	\
+		? xntoa( (cp)->co_remote_address )	\
 		: "<no address>"
 
 status_e 		conn_init() ;
