@@ -26,6 +26,8 @@
 #define TRUE  							1
 #endif
 
+#define FERROR 2
+
 #define NUL                      '\0'
 
 #define ES_NOMEM						"out of memory"
@@ -37,7 +39,11 @@
 #define RUSAGE_NULL					((struct rusage *)0)
 #define TIMEVAL_NULL					((struct timeval *)0)
 #define ITIMERVAL_NULL				((struct itimerval *)0)
+#ifdef INET6
+#define SOCKADDRIN_NULL             ((struct sockaddr_in6 *)0)
+#else
 #define SOCKADDRIN_NULL				((struct sockaddr_in *)0)
+#endif
 
 #define PRIVATE						static
 
