@@ -4,7 +4,7 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: init.c,v 1.2 1995/09/10 15:42:38 chuck Exp $" ;
+static char RCSid[] = "$Id: init.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include <sys/types.h>
 #include <sys/time.h>
@@ -24,9 +24,8 @@ static char RCSid[] = "$Id: init.c,v 1.2 1995/09/10 15:42:38 chuck Exp $" ;
 #include "config.h"
 #include "conf.h"
 
-#ifdef BSD		/* fix for getrlimit() problem in BSDI */
-#undef RLIMIT_NOFILE
-#include <unistd.h>
+#ifdef NO_GETDTABLESIZE
+#define getdtablesize()			64
 #endif
 
 struct module
