@@ -151,12 +151,20 @@ void int_init( ip, serp )
  */
 channel_s *int_newconn( ip, sinp, remote_socket )
 	struct intercept		*ip ;
+#ifdef INET6
+	struct sockaddr_in6	*sinp ;
+#else
 	struct sockaddr_in	*sinp ;
+#endif
 	int						remote_socket ;
 {
 	struct service			*sp			= SERVER_SERVICE( INT_SERVER( ip ) ) ;
 	int						socket_type = SVC_SOCKET_TYPE( sp ) ;
+#ifdef INET6
+	struct sockaddr_in6	*local		= INT_LOCALADDR( ip ) ;
+#else
 	struct sockaddr_in	*local		= INT_LOCALADDR( ip ) ;
+#endif
 	char 						*sid			= SVC_ID( sp ) ;
    channel_s				*chp ;
    int						sd ;
@@ -167,7 +175,11 @@ channel_s *int_newconn( ip, sinp, remote_socket )
 	 *
 	 * XXX:	should we use SC_PROTOVAL to explicitly specify the protocol ?
     */
+#ifdef INET6
+   if ( ( sd = socket( AF_INET6, socket_type, 0 ) ) == -1 )
+#else
    if ( ( sd = socket( AF_INET, socket_type, 0 ) ) == -1 )
+#endif
    {
       msg( LOG_ERR, func,"(intercepting %s) socket creation failed: %m", sid ) ;
       return( CHANNEL_NULL ) ;
@@ -216,7 +228,11 @@ channel_s *int_newconn( ip, sinp, remote_socket )
  */
 channel_s *int_lookupconn( ip, sinp, addr_checked )
 	struct intercept		*ip ;
+#ifdef INET6
+   struct sockaddr_in6	*sinp ;
+#else
    struct sockaddr_in	*sinp ;
+#endif
    bool_int					*addr_checked ;
 {
    register unsigned		u ;
@@ -228,12 +244,21 @@ channel_s *int_lookupconn( ip, sinp, addr_checked )
    {
       register channel_s *chp = CHP( pset_pointer( conntab, u ) ) ;
 
+#ifdef INET6
+      if ( memcmp(&chp->ch_from.sin6_addr, &sinp->sin6_addr, sizeof(sinp->sin6_addr)) == 0 )
+      {
+         *addr_checked = TRUE ;
+         if ( chp->ch_from.sin6_port == sinp->sin6_port )
+            return( chp ) ;
+      }
+#else
       if ( chp->ch_from.sin_addr.s_addr == sinp->sin_addr.s_addr )
       {
          *addr_checked = TRUE ;
          if ( chp->ch_from.sin_port == sinp->sin_port )
             return( chp ) ;
       }
+#endif
    }
    return( CHANNEL_NULL ) ;
 }
