@@ -49,6 +49,21 @@ int main( argc, argv )
 	init_daemon( argc, argv ) ;
 	init_services() ;
 
+	syslog( LOG_NOTICE, "%s started with ", VERSION );
+#ifdef INET6
+	syslog( LOG_NOTICE, "ipv6 ");
+#endif
+#ifdef LIBWRAP
+	syslog( LOG_NOTICE, "libwrap ");
+#endif
+#ifdef HAVE_LOADAVG
+	syslog( LOG_NOTICE, "loadavg "); 
+#endif
+#if !defined(INET6) && !defined(LIBWRAP) && !defined(HAVE_LOADAVG)
+	syslog( LOG_NOTICE, "no ");
+#endif
+	syslog( LOG_NOTICE, "options compiled in.");
+
 	msg( LOG_NOTICE, func, "Started working: %d available service%s",
 		ps.rws.available_services,
 			( ps.rws.available_services > 1 ) ? "s" : "" ) ;
