@@ -15,12 +15,12 @@
  * Macros about masks
  */
 
-#define mask_t unsigned
+#define mask_t unsigned long long
 
 #define MASK_EMPTY				0
 #define MASK_NULL				((mask_t *)0)
 
-#define MASK( v )				( 1 << ( (v)-1 ) )
+#define MASK( v )				( (unsigned long long)1 << ( (v)-1 ) )
 
 #define M_CLEAR_ALL( mask )			(mask) = 0
 #define M_ASSIGN( mask1, mask2 )		(mask1) = (mask2)
