@@ -70,6 +70,8 @@ status_e per_source_parser() ;
 status_e groups_parser() ;
 status_e banner_success_parser() ;
 status_e banner_fail_parser() ;
+status_e cps_parser() ;
+status_e enabled_parser() ;
 #ifdef HAVE_LOADAVG
 status_e max_load_parser() ;
 #endif
@@ -118,6 +120,7 @@ static struct attribute service_attributes[] =
    	{ "groups",         A_GROUPS,         1,  groups_parser          },
    	{ "banner_success", A_BANNER_SUCCESS, 1,  banner_success_parser  },
    	{ "banner_fail",    A_BANNER_FAIL,    1,  banner_fail_parser     },
+	{ "cps",            A_CPS,            2,  cps_parser             },
 #ifdef HAVE_LOADAVG
    	{ "max_load",       A_MAX_LOAD,       1,  max_load_parser        },
 #endif
@@ -141,6 +144,8 @@ static struct attribute default_attributes[] =
       { "groups",          A_GROUPS,          1,   groups_parser         },
       { "banner_success",  A_BANNER_SUCCESS,  1,   banner_success_parser },
       { "banner_fail",     A_BANNER_FAIL,     1,   banner_fail_parser    },
+      { "cps",             A_CPS,             2,   cps_parser            },
+      { "enabled",         A_ENABLED,        -2,   enabled_parser        },
 #ifdef HAVE_LOADAVG
       { "max_load",        A_MAX_LOAD,       1,    max_load_parser       },
 #endif
@@ -407,7 +412,7 @@ PRIVATE void get_service_entry( fd, sconfs, name, defaults )
    	{
    		if( (SCP(pset_pointer(sconfs, u))->sc_bind_addr != NULL) &&
    			(scp->sc_bind_addr != NULL) && 
-   			(SCP(pset_pointer(sconfs, u))->sc_bind_addr->s_addr != scp->sc_bind_addr->s_addr) )
+    		(memcmp(&SCP(pset_pointer(sconfs, u))->sc_bind_addr, &scp->sc_bind_addr, sizeof( scp->sc_bind_addr ) ) != 0) )
    		{
    			continue;
    		}
