@@ -16,6 +16,7 @@ static char RCSid[] = "$Id: parsers.c,v 1.5 1999/10/14 23:46:27 bbraun Exp $" ;
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <unistd.h>
+#include <limits.h> /* For LONG_MIN and LONG_MAX */
 
 #if defined(hpux) && !defined(X_OK)
 #define X_OK 1
@@ -405,6 +406,45 @@ status_e per_source_parser( values, scp, op )
 	return(OK);
 }
 
+status_e cps_parser( values, scp, op )
+	pset_h values ;
+	struct service_config *scp ;
+	enum assign_op op ;
+{
+	char *cps = (char *) pset_pointer(values, 0);
+	char *waittime = (char *) pset_pointer(values, 1);
+
+	if( cps == NULL || waittime == NULL ) {
+		parsemsg(LOG_ERR, "cps_parser", "NULL options specified in cps\n");
+		return( FAILED );
+	}
+	scp->sc_time_conn_max = strtol(cps, NULL, 10);
+	if( scp->sc_time_conn_max == LONG_MIN || 
+		scp->sc_time_conn_max == LONG_MAX ) {
+
+		parsemsg(LOG_ERR, "cps_parser", "cps arguement not a number\n");
+		scp->sc_time_conn_max = 0;
+		scp->sc_time_wait = 0;
+		return(FAILED);
+	}
+	scp->sc_time_wait = strtol(waittime, NULL, 10);
+	if( scp->sc_time_wait == LONG_MIN || scp->sc_time_wait == LONG_MAX ) {
+		parsemsg(LOG_ERR, "cps_parser", "cps time arguement not a number\n");
+		scp->sc_time_conn_max = 0;
+		scp->sc_time_wait = 0;
+		return(FAILED);
+	}
+
+	if( scp->sc_time_conn_max < 0 || scp->sc_time_wait < 0 ) {
+		parsemsg(LOG_ERR, "cps_parser", "cps arguements invalid\n");
+		scp->sc_time_conn_max = 0;
+		scp->sc_time_wait = 0;
+		return(FAILED);
+	}
+
+	return(OK);
+}
+
 status_e id_parser( values, scp, op )
 	pset_h values ;
 	struct service_config *scp ;
@@ -590,6 +630,27 @@ status_e disabled_parser( values, scp, op )
 }
 
 
+status_e enabled_parser( values, scp, op )
+	pset_h values ;
+	struct service_config *scp ;
+	enum assign_op op ;
+{
+	register unsigned u ;
+	char *func = "enabled_parser" ;
+
+	NEW_SET( scp->sc_enabled, pset_count( values ), 0 ) ;
+	
+	for ( u = 0 ; u < pset_count( values ) ; u++ )
+	{
+		char *name = (char *) pset_pointer( values, u ) ;
+
+		if ( add_new_string( scp->sc_enabled, name ) == FAILED )
+			return( OK ) ;
+syslog(LOG_ERR, "added %s\n", name);
+	}
+	return( OK ) ;
+}
+
 /*
  * Interpret a number of the form: <num>[m|M|k|K]
  * m and M mean megabytes, k and K mean kilobytes, nothing means bytes
@@ -861,12 +922,22 @@ status_e no_access_parser( values, scp, op )
 
 status_e banner_parser(pset_h values, struct service_config *scp, enum assign_op op)
 {
+	char *func = "banner_parser";
+
 	if( pset_pointer(values, 0) == NULL )
 	{
+		msg(LOG_ERR, func, "%s: pset_pointer returned null\n", func);
 		return FAILED;
 	}
 
-	scp->sc_banner = (char *)malloc(strlen(pset_pointer(values,0)));
+	scp->sc_banner = (char *)malloc(strlen(pset_pointer(values,0))+1);
+	if( scp->sc_banner == NULL ) {
+		msg(LOG_ERR, func, "%s: Out of memory\n", func);
+		return FAILED;
+	}
+
+	bzero(scp->sc_banner, strlen(pset_pointer(values,0))+1);
+
 	strcpy(scp->sc_banner, pset_pointer(values,0));
 
 	return OK;
@@ -874,12 +945,20 @@ status_e banner_parser(pset_h values, struct service_config *scp, enum assign_op
 
 status_e banner_success_parser(pset_h values, struct service_config *scp, enum assign_op op)
 {
-	if( pset_pointer(values, 0) == NULL )
-		return FAILED;
+	char *func = "banner_success_parser";
 
-	scp->sc_banner_success = (char *)malloc(strlen(pset_pointer(values,0)));
-	if( scp->sc_banner_success == NULL )
+	if( pset_pointer(values, 0) == NULL ) {
+		msg(LOG_ERR, func, "%s: pset_pointer returned null.\n", func);
 		return FAILED;
+	}
+
+	scp->sc_banner_success = (char *)malloc(strlen(pset_pointer(values,0))+1);
+	if( scp->sc_banner_success == NULL ) {
+		msg(LOG_ERR, func, "%s: Out of memory.\n", func);
+		return FAILED;
+	}
+
+	bzero(scp->sc_banner_success, strlen(pset_pointer(values,0))+1);
 
 	strcpy(scp->sc_banner_success, pset_pointer(values,0));
 	return OK;
@@ -887,14 +966,22 @@ status_e banner_success_parser(pset_h values, struct service_config *scp, enum a
 
 status_e banner_fail_parser(pset_h values, struct service_config *scp, enum assign_op op)
 {
-	if( pset_pointer(values, 0) == NULL )
-		return FAILED;
+	char *func = "banner_fail_parser";
 
-	scp->sc_banner_fail = (char *)malloc(strlen(pset_pointer(values,0)));
-	if( scp->sc_banner_success == NULL )
+	if( pset_pointer(values, 0) == NULL ) {
+		msg(LOG_ERR, func, "%s: pset_pointer returned NULL\n", func);
 		return FAILED;
+	}
+
+	scp->sc_banner_fail = (char *)malloc(strlen(pset_pointer(values,0))+1);
+	if( scp->sc_banner_fail == NULL ) {
+		msg(LOG_ERR, func, "%s: Out of memory\n", func);
+		return FAILED;
+	}
+	bzero(scp->sc_banner_fail, strlen(pset_pointer(values,0))+1);
 	
 	strcpy(scp->sc_banner_fail, pset_pointer(values,0));
+
 	return OK;
 }
 
@@ -923,6 +1010,56 @@ status_e redir_parser(pset_h values, struct service_config *scp, enum assign_op
 	char *adr = (char *)pset_pointer(values, 0);
 	char *foo;
 	char *uptmp;
+#ifdef INET6
+	char v6addr[46];
+	struct addrinfo hints, *res, *ressave;
+	int n;
+	char *func="redir_parser";
+	char *xaddrmap(char *, char *);
+
+	uptmp = pset_pointer(values, 1);
+	scp->sc_redir_port = strtol(uptmp, NULL, 10);
+
+	scp->sc_redir_addr = (struct in6_addr *)malloc(sizeof(struct in6_addr));
+
+	if( scp->sc_redir_addr == NULL )
+	{
+		msg(LOG_ERR, func, "%s: can't allocate space for redir addr\n", func);
+		return FAILED;
+	}
+
+	bzero(&hints, sizeof(hints));
+	hints.ai_flags = AI_CANONNAME;
+	hints.ai_family = AF_INET6;
+
+	if( (n = getaddrinfo(adr, NULL, &hints, &res)) < 0 ) {
+		hints.ai_family = AF_INET;
+		if( (n = getaddrinfo(adr, NULL, &hints, &res ) ) < 0 ) {
+			msg(LOG_ERR, func, "%s: Trying IPv4 name failed\n", func);
+			return FAILED;
+		}
+
+		bzero(v6addr, 46);
+		hints.ai_family = AF_INET6;
+		xaddrmap(inet_ntoa( ((struct sockaddr_in *)(res->ai_addr))->sin_addr), v6addr);
+		freeaddrinfo(res);
+
+		if( (n = getaddrinfo(v6addr, NULL, &hints, &res)) < 0 ) {
+			msg(LOG_ERR, func, "%s: bad address", func);
+			return FAILED;
+		}
+	}
+
+	if( (res == NULL) || (res->ai_addr == NULL) ) {
+		msg(LOG_ERR, func, "%s: no addresses returned", func);
+		return FAILED;
+	}
+		
+	memcpy(scp->sc_redir_addr, &((struct sockaddr_in6 *)res->ai_addr)->sin6_addr, sizeof(struct in6_addr));
+
+	return OK;
+
+#else
 	unsigned long tempaddr;
 	struct hostent *he;
 
@@ -969,6 +1106,7 @@ status_e redir_parser(pset_h values, struct service_config *scp, enum assign_op
 	}
 
 	return OK;
+#endif /* INET6 */
 }
 
 status_e bind_parser( pset_h values, struct service_config *scp, enum assign_op op)
@@ -976,6 +1114,53 @@ status_e bind_parser( pset_h values, struct service_config *scp, enum assign_op
 	char *adr = (char *)pset_pointer(values, 0);
 	char *foo;
 	unsigned long tempaddr;
+	char *func = "bind_parser";
+#ifdef INET6
+	int n;
+	char v6addr[46];
+	struct addrinfo hints, *res;
+	char *xaddrmap(char *, char *);
+
+	scp->sc_bind_addr = (struct in6_addr *)malloc(sizeof(struct in6_addr));
+
+	if( scp->sc_bind_addr == NULL )
+	{
+		msg(LOG_ERR, func, "%s: can't allocate space for bind addr\n", func);
+		return FAILED;
+	}
+
+	bzero(&hints, sizeof(hints));
+	hints.ai_flags = AI_CANONNAME;
+	hints.ai_family = AF_INET6;
+
+	if( (n = getaddrinfo(adr, NULL, &hints, &res ) ) < 0 ) {
+		hints.ai_family = AF_INET;
+		if( (n = getaddrinfo(adr, NULL, &hints, &res ) ) < 0 ) {
+			msg(LOG_ERR, func, "%s: Trying IPv4 name failed\n", func);
+			return FAILED;
+		}
+
+		bzero(v6addr, 46);
+		hints.ai_family = AF_INET6;
+		xaddrmap(inet_ntoa( ((struct sockaddr_in *)(res->ai_addr))->sin_addr), v6addr);
+		freeaddrinfo(res);
+
+		if( (n = getaddrinfo(v6addr, NULL, &hints, &res ) ) < 0 ) {
+			msg(LOG_ERR, func, "%s: bad address", func);
+			return FAILED;
+		}
+	}
+
+	if( ( res == NULL ) || ( res->ai_addr == NULL ) ) {
+		msg(LOG_ERR, func, "%s: no addresses returned", func);
+		return FAILED;
+	}
+
+	memcpy(scp->sc_bind_addr, &((struct sockaddr_in6 *)(res->ai_addr))->sin6_addr, sizeof(struct in6_addr));
+
+	freeaddrinfo(res);
+	return OK;
+#else
 	struct hostent *he;
 
 	scp->sc_bind_addr = (struct in_addr *)malloc(sizeof(struct in_addr));
@@ -1017,6 +1202,7 @@ status_e bind_parser( pset_h values, struct service_config *scp, enum assign_op
 	}
 	
 	return OK;
+#endif /* INET6 */
 	
 }
 
