@@ -36,6 +36,7 @@
 #define ST_INTERNAL     2
 #define ST_UNLISTED     3
 #define ST_SPECIAL	4
+#define ST_DISABLED     5 /* Note, this is *only* used by the enable option */
 
 /*
  * Service flags
@@ -112,17 +113,28 @@ struct service_config
    struct log         sc_log ;
    struct rpc_data    sc_rd ;
    pset_h             sc_disabled ;         /* used only by the default entry */
+   pset_h             sc_enabled ;         /* used only by the default entry */
    struct environment sc_environment ;
    builtin_s         *sc_builtin ;
-   struct in_addr    *sc_redir_addr ;
    unsigned           sc_redir_port;
+#ifdef INET6
+   struct in6_addr   *sc_redir_addr ;
+   struct in6_addr   *sc_bind_addr ;
+#else
+   struct in_addr    *sc_redir_addr ;
    struct in_addr    *sc_bind_addr ;
+#endif
    char              *sc_banner ;
    int                sc_per_source ;
    boolean_e          sc_groups ;
    char              *sc_banner_success ;
    char              *sc_banner_fail ;
    double             sc_max_load ;
+   time_t             sc_time_limit ;
+   int                sc_time_conn ;
+   int                sc_time_conn_max ;
+   time_t             sc_time_wait ;
+   time_t             sc_time_reenable ;
 } ;
 
 #define SCP( p ) ((struct service_config *)(p))
@@ -173,6 +185,9 @@ struct service_config
 #define SC_IS_SPECIAL( scp )     ( M_IS_SET( (scp)->sc_type, ST_SPECIAL ) )
 #define SC_IS_UNLISTED( scp )    ( M_IS_SET( (scp)->sc_type, ST_UNLISTED ) )
 #define SC_IS_INTERCEPTED( scp ) ( M_IS_SET( (scp)->sc_xflags, SF_INTERCEPT ) )
+#define SC_IS_DISABLED( scp )    ( M_IS_SET( (scp)->sc_xflags, ST_DISABLED ) )
+#define SC_DISABLE(scp)          ( M_SET( (scp)->sc_xflags, ST_DISABLED ) )
+#define SC_ENABLE(scp)           ( M_CLEAR( (scp)->sc_xflags, ST_DISABLED ) )
 
 #define LOGS_USERID( scp, flags ) \
 	( M_IS_SET( (scp)->flags, LO_USERID ) && SC_ACCEPTS_CONNECTIONS( scp ) )
