@@ -188,7 +188,11 @@ PRIVATE status_e set_fd_modes( sp )
 PRIVATE status_e activate_rpc( sp )
    register struct service *sp ;
 {
+#ifdef INET6
+	struct sockaddr_in6    sin ;
+#else
 	struct sockaddr_in     sin ;
+#endif
 	int                    sin_len ;
 	unsigned long          vers ;
 	struct service_config *scp = SVC_CONF( sp ) ;
@@ -200,12 +204,22 @@ PRIVATE status_e activate_rpc( sp )
 
 	CLEAR( sin ) ;
 
+#ifdef INET6
+	sin.sin6_family = AF_INET6 ;
+	if( scp->sc_bind_addr != NULL )
+		memcpy(&(sin.sin6_addr), &scp->sc_bind_addr, sizeof(sin.sin6_addr));
+	else
+		bzero(&sin.sin6_addr, sizeof(sin.sin6_addr));
+		
+	sin.sin6_port = 0 ;
+#else
 	sin.sin_family = AF_INET ;
 	if( scp->sc_bind_addr != NULL )
 		sin.sin_addr.s_addr = scp->sc_bind_addr->s_addr ;
 	else
 		sin.sin_addr.s_addr = htonl( INADDR_ANY ) ;
 	sin.sin_port = 0 ;         /* let the system give us a port */
+#endif /* INET6 */
 
 	if ( bind( sd, SA( &sin ), sizeof( sin ) ) == -1 )
 	{
@@ -223,7 +237,11 @@ PRIVATE status_e activate_rpc( sp )
 				"getsockname failed (%m). service = %s", sid ) ;
 		return( FAILED ) ;
 	}
+#ifdef INET6
+	sc_set_port( scp, ntohs( sin.sin6_port ) ) ;
+#else
 	sc_set_port( scp, ntohs( sin.sin_port ) ) ;
+#endif
 
 	/*
 	 * Try to register as many versions as possible
@@ -250,13 +268,30 @@ PRIVATE status_e activate_rpc( sp )
 PRIVATE status_e activate_normal( sp )
    register struct service *sp ;
 {
+#ifdef INET6
+	struct sockaddr_in6     sin ;
+#else
 	struct sockaddr_in		sin ;
+#endif
 	int							sd					= SVC_FD( sp ) ;
 	struct service_config	*scp				= SVC_CONF( sp ) ;
 	unsigned short				service_port	= SC_PORT( scp ) ;
 	char							*sid				= SC_ID( scp ) ;
 	char							*func				= "activate_normal" ;
 
+#ifdef INET6
+	bzero(&sin, sizeof(sin));
+
+	sin.sin6_family = AF_INET6;
+	sin.sin6_port = htons( service_port );
+	bzero( &(sin.sin6_addr), sizeof( sin.sin6_addr ) );
+
+	if( scp->sc_bind_addr != NULL )
+		memcpy(&sin.sin6_addr, scp->sc_bind_addr, sizeof(sin.sin6_addr));
+	else
+		bzero(&sin.sin6_addr, sizeof(sin.sin6_addr));
+
+#else
 	CLEAR( sin ) ;
 
 	sin.sin_family = AF_INET ;
@@ -265,6 +300,7 @@ PRIVATE status_e activate_normal( sp )
 	else
 		sin.sin_addr.s_addr = htonl( INADDR_ANY ) ;
 	sin.sin_port = htons( service_port ) ;
+#endif
 
 	if ( reuse_option || SC_REUSE_ADDRESS( scp ) )
 	{
@@ -302,8 +338,13 @@ status_e svc_activate( sp )
    char							*func = "svc_activate" ;
 	voidfunc						get_shutdown_by_name() ;
 
+#ifdef INET6
+	sp->svc_fd = socket( AF_INET6, 
+								SC_SOCKET_TYPE( scp ), SC_PROTOVAL( scp ) ) ;
+#else
 	sp->svc_fd = socket( AF_INET, 
 								SC_SOCKET_TYPE( scp ), SC_PROTOVAL( scp ) ) ;
+#endif
 
    if ( sp->svc_fd == -1 )
    {
@@ -674,14 +715,23 @@ status_e svc_access_control( sp, cp )
 		 */
 		if ( result == AC_ADDRESS && SVC_SOCKET_TYPE( sp ) == SOCK_DGRAM )
 		{
+#ifdef INET6
+			register struct sockaddr_in6 *sinp = conn_address( cp ) ;
+			register struct sockaddr_in6 *last = &sp->svc_last_dgram_addr ;
+#else
 			register struct sockaddr_in *sinp = conn_address( cp ) ;
 			register struct sockaddr_in *last = &sp->svc_last_dgram_addr ;
+#endif
 			time_t current_time ;
 
 			(void) time( &current_time ) ;
 
+#ifdef INET6
+			if ( IN6_ARE_ADDR_EQUAL(&(sinp->sin6_addr), &(last->sin6_addr)) && sinp->sin6_port == last->sin6_port )
+#else
 			if ( sinp->sin_addr.s_addr == last->sin_addr.s_addr &&
 														sinp->sin_port == last->sin_port )
+#endif
 			{
 				if ( current_time - sp->svc_last_dgram_time <= DGRAM_IGNORE_TIME )
 					report_failure = FALSE ;
