@@ -64,7 +64,11 @@ struct service
 	 * These fields are used to avoid generating too many messages when
 	 * receiving datagrams from a bad address.
 	 */
+#ifdef INET6
+	struct sockaddr_in6                     svc_last_dgram_addr ;
+#else
 	struct sockaddr_in                      svc_last_dgram_addr ;
+#endif
 	time_t                                  svc_last_dgram_time ;
 	xlog_h                                  svc_log ;
 } ;
