@@ -175,8 +175,12 @@ PRIVATE status_e handle_io( iter, chp, maskp, iofunc )
 			if ( debug.on )
 				msg( LOG_DEBUG, func,
 					"Closing channel to %s,%d using sockets %d(l),%d(r)",
-						inet_ntoa( chp->ch_from.sin_addr ),
+						xntoa( chp->ch_from ),
+#ifdef INET6
+							ntohs( chp->ch_from.sin6_port ),
+#else
 							ntohs( chp->ch_from.sin_port ),
+#endif
 								chp->ch_local_socket, chp->ch_remote_socket ) ;
 
 			FD_CLR( chp->ch_local_socket, maskp ) ;
@@ -195,7 +199,13 @@ PRIVATE void connection_request( ip, chpp )
 	struct intercept	*ip ;
 	channel_s			**chpp ;
 {
+#ifdef INET6
+	struct sockaddr_in6	sin ;
+	unsigned short *port = &sin.sin6_port;
+#else
 	struct sockaddr_in	sin ;
+	unsigned short *port = &sin.sin_port;
+#endif
 	int						sin_len = sizeof( sin ) ;
 	channel_s				*chp ;
 	int						sd ;
@@ -211,7 +221,7 @@ PRIVATE void connection_request( ip, chpp )
 
 	if ( debug.on )
 		msg( LOG_DEBUG, func, "connection request from %s,%d",
-			inet_ntoa( sin.sin_addr ), ntohs( sin.sin_port ) ) ;
+			xntoa( sin ), ntohs( *port ) ) ;
 
 	chp = int_lookupconn( ip, &sin, &addr_checked ) ;
 	if ( chp == NULL )
@@ -263,7 +273,7 @@ PRIVATE void connection_request( ip, chpp )
 	else
 		msg( LOG_ERR, func,
 			"Received another connection request from %s,%d",
-				inet_ntoa( sin.sin_addr ), ntohs( sin.sin_port ) ) ;
+				xntoa( sin ), ntohs( *port ) ) ;
 }
 
 
